#!/usr/bin/perl
#
# $Id: check_wmiproxy.pl 9 2009-10-26 17:23:32Z brentashley $
#

#
# check_wmiproxy.pl
#
#   http://wmi-proxy.sourceforge.net
#
#   23 Oct 2009 released by KnowledgeWorx - http://www.kwx.ca 
#   under BSD license
# 
#   calls proxy service to perform wmi queries
#   requires kwx proxy service running on win box
#   requires conf file containing name = value lines 
#   (file location specified in kwx.conf as wmiproxyconf)
#      conf vars are: protocol, host, path, namespace, username, password
#  

my $conf = '../config/wmiproxy.conf';

use SOAP::Lite on_action => sub { "$_[0]/$_[1]"; }; 
import SOAP::Data 'name', 'value';
use LWP::UserAgent;

my $debug = 0;
if ($debug > 0) {
    LWP::Debug::level('+');
    SOAP::Lite->import(+trace => 'debug') if ($debug > 1);
}

use strict;
no strict 'refs';

my %ERRORS = (
    OK => 0,
    WARNING => 1,
    CRITICAL => 2,
    UNKNOWN => 3,
    DEPENDENT => 4
);

# parameters 
my $action = lc shift;
my $host = shift;
my $options = shift;

unless ($host) {
    syntax();
    exit $ERRORS{'CRITICAL'};
}

# proxy config
my $proxyProto = confVar($conf,'protocol');
my $proxyHost = confVar($conf,'host');
my $proxyPath = confVar($conf,'path');
my $proxyNamespace = confVar($conf,'namespace');

# create SOAP object
my @credentials = ($proxyHost, '', confVar($conf,'username'), confVar($conf,'password') );
my @ua_args = (keep_alive => 1);
my $schema_ua = LWP::UserAgent->new(@ua_args);
$schema_ua->credentials(@credentials);
my $soap = SOAP::Lite
	-> proxy($proxyProto . '://' . $proxyHost . $proxyPath, @ua_args, credentials => \@credentials)
	-> uri($proxyNamespace);
$soap->schema_useragent($schema_ua);

unless ($soap) {
    print "Soap service failed to initialize\n";
    exit $ERRORS{'UNKNOWN'};
}

#
# dispatch actions
# each action block must have all paths lead to exit
#
# wmi proxy returns array of [code,output] 
# codes are OK FAILED ERROR UNKNOWN
#

my %opt = optionsHash($options);

if ( $action eq 'uptime' ) {

    my $warn = $opt{'w'} || 12;
    my $crit = $opt{'c'} || 2;

    my $result = $soap->Uptime(
        SOAP::Data->name('host',$host )
    )->result->{'string'};
    my $code = $result->[0];
    my $output = $result->[1];

    print "$output\n";
    if( $code eq 'OK' ) {
        exit $ERRORS{'CRITICAL'} if( $output < $crit );
        exit $ERRORS{'WARNING'}  if( $output < $warn );
	exit $ERRORS{'OK'};
    }
    exit $ERRORS{'UNKNOWN'} if( $code eq 'UNKNOWN' );

    # error or failure
    exit $ERRORS{'CRITICAL'};
}

if ( $action eq 'diskfree' ) {

    my $drive = $opt{'d'} || 'c';
    my $warn = $opt{'w'} || 15;
    my $crit = $opt{'c'} || 10;

    my $result = $soap->FixedDiskPctFree(
        SOAP::Data->name('host',$host),
        SOAP::Data->name('drive',$drive)
    )->result->{'string'};
    my $code = $result->[0];
    my $output = $result->[1];

    print "$output\n";
    if( $code eq 'OK' ) {
        exit $ERRORS{'CRITICAL'} if( $output < $crit );
        exit $ERRORS{'WARNING'}  if( $output < $warn );
	exit $ERRORS{'OK'};
    }
    exit $ERRORS{'UNKNOWN'} if( $code eq 'UNKNOWN' );

    # error or failure
    exit $ERRORS{'CRITICAL'};
}

if ( $action eq 'pingfromhost' ) {

    my $dest = $opt{'d'};
    my $warn = $opt{'w'} || 700;
    my $crit = $opt{'c'} || 1000;

    my $result = $soap->PingHostToDestination(
        SOAP::Data->name('host',$host),
        SOAP::Data->name('destination',$dest)
    )->result->{'string'};
    my $code = $result->[0];
    my $output = $result->[1];

    print "$output\n";
    if( $code eq 'OK' ) {
        exit $ERRORS{'CRITICAL'} if( $output > $crit );
        exit $ERRORS{'WARNING'}  if( $output > $warn );
        exit $ERRORS{'OK'};
    }
    exit $ERRORS{'UNKNOWN'} if( $code eq 'UNKNOWN' );

    # error or failure
    exit $ERRORS{'CRITICAL'};
}

if ( $action eq 'exchangequeues' ) {

    my $warn = $opt{'w'} || 50;
    my $crit = $opt{'c'} || 100;

    my $result = $soap->ExchangeQueues(
        SOAP::Data->name('host',$host),
    )->result->{'string'};
    my $code = $result->[0];
    my $output = $result->[1];

    chomp $output;
    my ($qlen) = $output =~ /has (\d+) messages/;

    print "$output\n";
    if( $code eq 'OK' ) {
        exit $ERRORS{'CRITICAL'} if( $qlen > $crit );
        exit $ERRORS{'WARNING'}  if( $qlen > $warn );
        exit $ERRORS{'OK'};
    }
    exit $ERRORS{'UNKNOWN'} if( $code eq 'UNKNOWN' );

    # error or failure
    exit $ERRORS{'CRITICAL'};

}

if ( $action eq 'servicecheck' ) {

    my $service = $opt{'service'};
    my $state = $opt{'state'} || 'running';

    my $result = $soap->ServiceCheck(
        SOAP::Data->name('host',$host),
        SOAP::Data->name('servicename',$service)
    )->result->{'string'};
    my $code = $result->[0];
    my $output = $result->[1];

    print "$output\n";
    if( $code eq 'OK' ) {
        exit $ERRORS{'OK'} if (lc $output eq lc $state);
        exit $ERRORS{'CRITICAL'};
    }
    exit $ERRORS{'UNKNOWN'} if( $code eq 'UNKNOWN' );

    # error or failure
    exit $ERRORS{'CRITICAL'};
}

if ( $action eq 'memoryfree' ) {

    my $type = $opt{'t'} || 'virt';
    if (($type ne 'phys') && ($type ne 'virt')) {
        print "type must be phys or virt\n";
        exit $ERRORS{'CRITICAL'};
    }

    my $warn = $opt{'w'} || 15;
    my $crit = $opt{'c'} || 10;

    my $result = $soap->Memory(
        SOAP::Data->name('host',$host),
    )->result->{'string'};
    my $code = $result->[0];
    my $output = $result->[1];

    my ($freeP) = $output =~ /FreePhysicalMemory:(\d+)/; 
    my ($freeV) = $output =~ /FreeVirtualMemory:(\d+)/; 
    my ($totP) = $output =~ /TotalPhysicalMemory:(\d+)/; 
    my ($totV) = $output =~ /TotalVirtualMemorySize:(\d+)/; 

    my $percent;
    if( $type eq 'phys' ) {
        $percent = int((100 * $freeP) / $totP);
    } else { 
        $percent = int((100 * $freeV) / $totV);
    }

    print "$percent\n";
    if( $code eq 'OK' ) {
        exit $ERRORS{'CRITICAL'} if( $percent < $crit );
        exit $ERRORS{'WARNING'}  if( $percent < $warn );
        exit $ERRORS{'OK'};
    }
    exit $ERRORS{'UNKNOWN'} if( $code eq 'UNKNOWN' );

    # error or failure
    exit $ERRORS{'CRITICAL'};
}

if ( $action eq 'wmiquery' ) {

    my $query = $opt{'q'};

    my $result = $soap->Query(
        SOAP::Data->name('host',$host),
        SOAP::Data->name('query',$query)
    )->result->{'string'};
    my $code = $result->[0];
    my $output = $result->[1];

    print "$output\n";
    if( $code eq 'OK' ) {
        exit $ERRORS{'OK'};
    }
    exit $ERRORS{'UNKNOWN'} if( $code eq 'UNKNOWN' );

    # error or failure
    exit $ERRORS{'CRITICAL'};
}


# shouldn't get here if action blocks all exit properly.
print "Action [$action] not recognized.\n";
syntax();
exit $ERRORS{'CRITICAL'};

# end of main program

sub syntax {

    print <<'HELPTEXT'; 

  check_wmi_proxy:

    syntax: ./check_wmi_proxy action host options

        options is pipe-separated name=value pairs

  Available actions and their options and defaults: 

  uptime host "w=12|c=2" 
    returns hours
  diskfree host "d=c|w=15|c=10" 
    returns percent
  memoryfree host "t=[*virt|phys]|w=15|c=10" 
    returns percent
  pingfromhost host "d=destaddr|w=700|c=1000" 
    returns RTT in ms
  servicecheck host "service=servicename|state=[*running|stopped]" 
    returns state
  exchangequeues host "w=50|c=100" 
    returns string: 'primary-queue-name' has 0 messages
  wmiquery host "q=select * from win32_logicaldisk where deviceid='c:'"
    returns query result string

HELPTEXT

}

sub confVar {
    my ($conffile, $variable) = @_;

    # get variable from conf file
    my $name = '';
    my $value = '';
    open CFG, $conffile;
    while (<CFG> ){
        chomp;
        next if /^\s*#/;
        next if /^\s*$/;
        ($name, $value) = /^\s*(.*?)\s*=\s*(.*)/;
        last if ($name eq $variable);
    }
    close CFG;
    return $value;
}

sub optionsHash {
    my ($options) = @_;
    
    # split name=value pairs on pipe and build into hash
    my %hash;
    foreach my $pair ( split(/\|/, $options) ){
        # split on first = only 
        my ($name, $value) = $pair =~ /(.*?)=(.*)/; 
        $hash{$name} = $value;
    } 
    return %hash;
}

